#ifndef RECTANGLE_DETECTION_CONTOURANALYZER_H_
#define RECTANGLE_DETECTION_CONTOURANALYZER_H_
#include <string>
#include <opencv2/opencv.hpp>
#include "../UI.h"
#include "../UIListener.hpp"
#include <memory>

class ContourAnalyzer: public UIListener {
	std::shared_ptr<int> lastKey;
	bool cannyRec = false, bindBrightnessTobinLower = true, drawContours = false;
	std::shared_ptr<UI> ui;
	std::vector<std::vector<cv::Point>> contours;
	std::vector<std::vector<cv::Point>> inconsistentContours;
	std::vector<std::vector<cv::Point>> consistentContours;
	std::vector<std::vector<cv::Point>> prunedContours;
	std::vector<cv::Rect> bounds;
	std::vector<cv::RotatedRect> rotatedBounds;
	cv::Mat luminescenceMat;
	double brightness = 0;
	int lastBrightnessCheck = 60;

	void findRectangles();
	void preAnalyze();
public:
	int binLower = 54;

	int minSize = 0;
	int minSizeScale = 10;
	int cannyLower = 42;
	int cannyUpper = 3*cannyLower;

	int epsilon = 56;

	ContourAnalyzer(std::shared_ptr<UI> ui, std::shared_ptr<int> lastKey);
	virtual ~ContourAnalyzer();
	cv::Scalar selectColor { 0, 255, 0 };
	cv::Scalar fontColor { 0, 0, 255 };
	void analyze();
	void analyze(cv::Rect rect);
	void drawContoursOntoMat(cv::Mat &frame);
	std::string contourToString(std::vector<cv::Point>);
	void convertContoursToBounds();
	void convertContoursToRotatedBounds();
	void suggestCalcBrightness();
	double getBrightness() const;
	std::vector<cv::Rect> getBounds() const;
	std::vector<cv::RotatedRect> getRotatedBounds() const;
	void componentSetup();
};

#endif /* RECTANGLE_DETECTION_CONTOURANALYZER_H_ */
